# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-06-06 13:14
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('oauth2_provider', '0006_auto_20171214_2232'),
        ('oauth2_provider', '0008_idtoken'),
    ]

    operations = [
    ]
