# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-08-16 17:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oauth2_provider', '0012_accesstoken_id_token'),
    ]

    operations = [
        migrations.AlterField(
            model_name='application',
            name='redirect_uris',
            field=models.TextField(blank=True, help_text='Allowed URIs list, space separated'),
        ),
    ]
